package tud.st.dpf.robot.simulator.participants;

public class Robot {

	private String name;
	private float maxLoad;
	private float batteryLoad;
	private Product carriedObject;
	private Position currentPosition;
	private int maxSpeed;
	private Boolean broken = false;

	public Robot(String name, float maxLoad, int maxSpeed) {
		this.name = name;
		this.maxLoad = maxLoad;
		this.maxSpeed = maxSpeed;

		this.batteryLoad = 100;

		new Thread(new Runnable() {

			@Override
			public void run() {
				while (true) {
					try {
						Thread.sleep(250);
					} catch (Exception e) {
						e.printStackTrace();
					}

					setBatteryLoad(batteryLoad - 1);
				}
			}
		}).start();
	}

	public String getName() {
		return name;
	}

	public float getMaxLoad() {
		return this.maxLoad;
	}

	public float getBatteryLoad() {
		return batteryLoad;
	}

	public void setBatteryLoad(float batteryLoad) {
		this.batteryLoad = batteryLoad;

		if (this.batteryLoad <= 0)
			this.batteryLoad = 0;
		
		if(this.batteryLoad > 100)
			this.batteryLoad = 100;

		if (batteryLoad == 0)
			setBroken(true);
	}

	public void take(Product p) {
		if (p.getWeight() > maxLoad)
			setBroken(true);

		this.carriedObject = p;
	}

	public Product unloadObject() {
		Product result = carriedObject;
		carriedObject = null;
		return result;
	}
	
	public Product getCarriedObject(){
		return carriedObject;
	}

	public Position getCurrentPosition() {
		return currentPosition;
	}

	public void setCurrentPosition(Position currentPosition) {
		if (currentPosition != null)
			System.out.println("Robot " + getName() + " arrived at " + currentPosition.getName());

		this.currentPosition = currentPosition;
	}

	public int getMaxSpeed() {
		return maxSpeed;
	}

	public void driveTo(Position p, int speed) {
		setCurrentPosition(null);

		if (broken)
			return;
		
		System.out.println("Robot " + getName() + " drives with speed " + speed);

		new Thread(new Runnable() {
			@Override
			public void run() {
				try {
					Thread.sleep((int) ((1000.0 / speed) * 2000.0));
				} catch (Exception e) {
					e.printStackTrace();
				}

				setCurrentPosition(p);
			}
		}).start();
	}

	public boolean isBroken() {
		return broken;
	}

	private void setBroken(boolean isBroken) {
		broken = isBroken;

		if (broken)
			System.err.println("Robot " + getName() + " is broken");
		else
			System.out.println("Robot " + getName() + " was repaired");
	}
	
	public boolean canTakeTask(Product p){
		return true;
	}
}
